//Cvideo.h interface for Cvideo class
// Theo Schouten

#if !defined(cvideoh)
#define cvideoh

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VFWImageProcessor.h"

CImgDisplay dispvideo(1,1,"Life Video",0,3,FALSE, TRUE); // no rescale of intensities

class Cvideo {
private:
	CVFWImageProcessor m_VFWImageProc;
	CAPDRIVERCAPS Caps;
	BITMAPINFOHEADER bmi;
	bool disp;
	BITMAPINFO *Bitmap;
	ULONG Length;

	void doreset() {
		if(disp) {
			dispvideo.resize(v_width, v_height);
			dispvideo.show();
			m_VFWImageProc.EnablePreviewVideo(dispvideo.window,0,0);
		}else {
			m_VFWImageProc.DisablePreviewVideo();
			//dispvideo.resize(0,0); // gives error with Cimg 1.18
			dispvideo.close();
		}
	}

public:
	Cvideo(const bool initdisp){  // Live video on or off
		m_VFWImageProc.Initialize();
		m_VFWImageProc.DriverGetCaps(&Caps);
		//printf("source: %d format: %d display: %d\n",Caps.fHasDlgVideoSource, Caps.fHasDlgVideoFormat, Caps.fHasDlgVideoDisplay);
	
		bmi = m_VFWImageProc.GetBitmapInfoHeader();
		v_width=bmi.biWidth; v_height=bmi.biHeight;
		//printf("width: %d height: %d\n",bmi.biWidth,bmi.biHeight);
		Bitmap=NULL; Length=0;
		disp=TRUE; doreset();
		disp=initdisp;
		doreset();
	}
	~Cvideo(){
	}

	int v_width, v_height;

	void reinit(){                // to be called after doformat() window is closed
		if(Bitmap != NULL) { delete Bitmap; Bitmap=NULL;}
		Length=0;
		m_VFWImageProc.Destroy();
		m_VFWImageProc.Initialize();
		m_VFWImageProc.DriverGetCaps(&Caps);	
		bmi = m_VFWImageProc.GetBitmapInfoHeader();
		v_width=bmi.biWidth; v_height=bmi.biHeight;
		doreset();
	}

	void dodisp(const bool initdisp) { disp=initdisp; doreset();} // set live video on or off
	void dodisp() { disp = !disp; doreset();}  // toggle live video

	void dosource(){ 
		if(Caps.fHasDlgVideoSource) {
			m_VFWImageProc.DlgVideoSource(); doreset(); // video might be switched off
		}
	}
	void doformat(){ // open a dialog panel in a thread, reinit() should be called after panel is closed
		if(Caps.fHasDlgVideoFormat) {
			m_VFWImageProc.DlgVideoFormat(); doreset();
		}
	}

void docapture(CImg<uchar> &capimg){
	m_VFWImageProc.CaptureDIB(&Bitmap,Length,&Length); // capture DIB image
	if(Bitmap){ // something there
		LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER) Bitmap;
		if(capimg.width != lpbi->biWidth || capimg.height != lpbi->biHeight || capimg.depth != 1 || capimg.dim != 3){
			capimg.width = lpbi->biWidth; capimg.height = lpbi->biHeight; capimg.depth=1; capimg.dim=3;
			delete capimg.data; // capimg has different size than the captured image
			capimg.data = new uchar [capimg.size()];
		}
		int x,y;
		uchar *p  = (uchar *)lpbi + lpbi->biSize;

		/*for(y=capimg.height-1; y >= 0; y--) { // works because width is multiple of 4
			for(x=0; x <capimg.width; x++) { // b,g,r is windows order, srating from bottom
				capimg(x,y,0,2) = *p++; capimg(x,y,0,1) = *p++; capimg(x,y,0,0) = *p++;
			}
		}*/
		// a bit faster coding:
		int pr = (capimg.height)*capimg.width;
		int pg = pr + capimg.height * capimg.width;
		int pb = pg + capimg.height * capimg.width;
		for(y=capimg.height-1; y >= 0; y--) { // works because width is multiple of 4
			pr -= capimg.width; pg -= capimg.width; pb -= capimg.width;
			for(x=0; x <capimg.width; x++) { // b,g,r is windows order, starting from bottom
				capimg[pb+x] = *p++; capimg[pg+x] = *p++; capimg[pr+x] = *p++;
			}
		}
	}
	if(disp)m_VFWImageProc.EnablePreviewVideo(dispvideo.window,0,0); // restart live video

}

}; // end of class Cvideo
#endif // !defined(cvideoh)